#!/usr/bin/perl 
#
# printfax.pl 1.3.4, last changed 22/09/1997
#
# (c) 1996-1997 Horst F <horstf(al)weltentor.boerde.de>
#
# based on the script by
#   Heiko Schlittermann <heiko(al)lotte.sax.de>
# added some modification to run under perl5.002 by
#   Don Hayward <don(al)marinelab.sarasota.fl.us>
#
# NOTE: I'm neither a perl nor network expert (as well
# English isn't my natural language too)

#####
# This script could be part of a samba/mgetty based fax solution
# for Win* clients. 
#
# It is intended to be used in conjunction with ``Winsock Respond Daemon''
# written by  "Horst F" <horstf(al)weltentor.boerde.de>
#
# You can download the current version from
# http://www.boerde.de/~horstf/
#
# called from smb.conf:
#   print command = ( /usr/bin/printfax.pl %I %s %U %m; rm %s ) &
#
# History (changes since 10/28/1996):
#   [10/28/1996]
#   - one script for all perl versions since 5.0 (or earlier?)
#     (joining the scripts for 5.002 and for earlier versions)
#   - replace of 'localhost' with call of hostname(), hope
#     this fixes the problem with SunOS
#   [10/29/1996]
#   - the use of hostname() doesn't solve the problem with SunOS,
#     the problem was gethostbyname, but now it works
#   - start version numbering, now it's 1.3.2
#   [09/22/1997 1.3.3]
#   - minor bug fix:
#     use "require/import" instead of "use Sys::Hostname" because
#     "use" includes a file at compile time and so it includes
#     Sys::Hostname every time, not only for perl version < 5.002
#     (thanks to Simon Hyde <shyde(al)poboxes.com>)
#   [10/29/1997 1.3.4]
#   - reading faxnumbers from file in user's home dir or everywhere
#   - added debug output
####

    require 5.0;      # Don't know if required include files exist
                      # in earlier versions
    use strict;
    no strict "refs";
    use Socket;

    # this is only required for perl versions below 5.002
    if ( $] < 5.002 ) {
      require Sys::Hostname;
      import Sys::Hostname;
    }

    my($last, $tmp);

###
### CONFIGURATION Section
###

    my($port, $usedelay, $defaultdelay, $acct, $faxspool, $smbclient,
       $logsmbhostname, $usesmbuser, $msg_to, $msg_ignored,
       $msg_spooled, $msg_failed, $debug, $allowfile, $filechar,
       $version, $nosendondebug);

### printfax.pl version string
    $version = "printfax.pl 1.3.4";

### The port we'll connect to RESPOND
    $port = 5555;

### Use delay
    $usedelay = 0;

### Default delay for delayed faxes
    $defaultdelay = "18:00";

### The log file we'll write the accounting information to
### (It has to be writeable by the user samba is running as for faxes)
    $acct = "/var/spool/fax/outgoing/printfax.log";

### The faxspoolprogram (for mgetty+sendfax it's faxspool)
    $faxspool = "/usr/local/bin/faxspool";

### The smbclient program, sender is the faxsystem
    $smbclient = "/usr/local/bin/smbclient -U FAX";

### The secure path for binaries searched 
### (faxspool makes usage of the PATH environment!)
    $ENV{PATH}  = '/usr/local/bin:/usr/bin:/bin' .
                  ':/usr/local/sbin:/usr/sbin:/sbin' .
                  ':/usr/X11R6/bin';

### Use smb hostname for logging, set to 0 for inet hostname
### (ip address will be logged in both cases)
    $logsmbhostname = 1;

### use smbuser for returning mail, ignore what the user types in
### (smbuser will be allways used for logging)
    $usesmbuser = 0;

### spool result messages via smbclient
    $msg_to = "to";
    $msg_ignored = "ignored";
    $msg_spooled = "spooled";
    $msg_failed = "failed";

### allow file for faxnumbers
    $allowfile = 1;
    $filechar = '@';

### set $debug to 1 for debug messages and $nosendondebug to 1 if
### you don't want to spool files on debug
    $debug = 0;
    $nosendondebug = 1;

###
### NOTHING else to configure
###

### If called non interactivly ... use the logger to report errors

    -t STDERR 
        || open(STDERR, "|logger -tprintfax") 
        || die "$0: Can't open logger: $!\n";


### Make STDERR unbuffered
    $last = select(STDERR); $| = 1;
    select($last);


    if ($debug) {
	print STDERR "$version started with:";
	foreach $tmp (@ARGV) {
	    print STDERR " $tmp";
	}
	print STDERR "\n";
    }


### COMMAND line processing
    my($remote_ip, $faxfile, $smbuser, $smbhost, $faxfname);

    # 1.: The ip address (smb.conf: %I)
    $remote_ip = $ARGV[0];
    # 2.: The name of the faxfile with path (%s)
    $faxfile = $ARGV[1];
    # 3.: The samba session setup user (%U)
    # used for accounting and if RESPOND doesn't return the user name
    $smbuser = $ARGV[2];
    # 4.: The NetBIOS hostname (%m)
    $smbhost = $ARGV[3];

    # Strip any domain from smbhost if required
    # (sometimes %m gets the full qualified internet name)
    $smbhost =~ s/^([^.]+).*$/$1/;

    # Strip path from filename 
    ($faxfname = $faxfile) =~ s|.*/||;


### Establish the socket connection
    my($ip_remote, $remote, $proto);
    $proto = getprotobyname('tcp');

    # Since version 5.002 perl uses new ipc method
    if ( $] < 5.002 ) {
      # This is for perl versions below 5.002

      my($ip_local, $local);
      $ip_local = (gethostbyname(hostname()))[4];
      $local = &Socket::sockaddr_in(AF_INET, 0, unpack('C4', $ip_local));

# This does not work on some systems (gethostbyname doesn't work properly):
#      $ip_remote = (gethostbyname($remote_ip))[4] || 
#			die "$0: No host: $remote_ip; $!\n";
#      $remote = &Socket::sockaddr_in(AF_INET, $port, unpack('C4', $ip_remote));

# so we use this:
      $remote = &Socket::sockaddr_in(AF_INET, $port, split(/\./, $remote_ip));

      if ($debug) {
	print STDERR "try to connect to $remote_ip as ",
		     join ('.', unpack('C4', $ip_local)), "\n";
      }

      socket(S, AF_INET, SOCK_STREAM, $proto) ||
	die "$0: Can't get socket: $!\n";
      bind(S, $local) || die "$0: Can't bind socket: $!\n";

    } else { 
      # This must be used since perl version 5.002

# This seams to work:
      $ip_remote = inet_aton($remote_ip) || die "$0: No host: $remote_ip\n";
      $remote = sockaddr_in($port, $ip_remote);

# if not, try this:
#      $remote = sockaddr_in($port, pack('C4', split(/\./, $remote_ip)));

      if ($debug) {
	print STDERR "try to connect to ",
		     join('.', unpack('C4', $ip_remote)), "\n";
      }

      socket(S, PF_INET, SOCK_STREAM, $proto) 
        || die "$0: Can't get socket: $!\n";

    }
    connect(S, $remote) || die "$0: Can't connect: $!\n";


    if ($debug) {
	print STDERR "connected\n";
    }

### Get the needed information
    my($faxnum, $user, $faxreceiver, $fullname, $delayed);

    # Read FaxNr, User, Receivers name and Users fullname
    # as well as strip trailing \n or \r
    ($faxnum = <S>) =~ tr/\r\n//d;
    ($user = <S>) =~ tr/\r\n//d;
    ($faxreceiver = <S>) =~ tr/\r\n//d;
    ($fullname = <S>) =~ tr/\r\n//d;
    ($delayed = <S>) =~ tr/\r\n//d;

    if ($debug) {
	print STDERR "got faxnumber: $faxnum\n";
	print STDERR "got receiver : $faxreceiver\n";
	print STDERR "got sender   : $user\n";
	print STDERR "got fullname : $fullname\n";
	print STDERR "got delay    : $delayed\n";
    }

    # use default delay if respond's delay checkbox checked
    if ( $delayed eq 'delayed' ) { $delayed = $defaultdelay; }

    # Close the query connection
    close(S);

    # use samba user if no user is given
    $user = $smbuser unless $user;

    # allways use smbuser (look in the configuration section)
    $user = $smbuser if $usesmbuser;

    # user needs to lowercased too
    $user =~ tr/A-Z/a-z/;

    my(@faxnums, $gcos);

    # split space or comma delimited faxnums (for more then one receiver)
    @faxnums = split(/[ ,]+/, $faxnum); 


    # Retrieve additional information about the user (here: the users
    # fullname) and the date

    # Extract fullname from /etc/passwd 
    # only if we didn't get it from respond
    if ( $fullname eq '' ) {
        $gcos = (getpwnam($user))[6];
        $fullname = (split(/,/, $gcos))[0];
    }
    $fullname = $user unless $fullname;

### OK, do accounting

    # What says the clock?
    my($s, $m, $h, $dy, $mo, $yr);
    ($s, $m, $h, $dy, $mo, $yr) = (localtime(time))[0..5];
    $mo++;      # month is 0 based.

    my($host, $faxdest);

    # Log smb or inet hostname (look in the configuration section)
    if ( $logsmbhostname ) {

    # Use the NetBIOS name for accounting:
    $host = "$smbhost($remote_ip)";

    # Uppercase looks more NetBIOS like :-)
    # uncomment if you don't want this
    $host =~ tr/a-z/A-Z/;

    } else {

    # Use the internet name for accounting:
    $host = (gethostbyaddr($ip_remote,AF_INET))[0];

    # Strip domains from name
    $host =~ s/^([^.]+).*$/$1/;
    $host = ($host) ? "$host($remote_ip)" : $remote_ip;

    }

    # Open the accounting file
    open(ACCT, ">>$acct") || die "$0: Can't open `$acct': $!\n";

    # Make it unbuffered
    $last = select(ACCT); $| = 1;
    select($last);

    # Establish the message connection, 
    # no die if open fails
    if (!$debug || ($debug && ! -t STDERR)) {
	open(SF,"|$smbclient -M $smbhost");
	$last = select(SF); $| = 1; select($last);
    }


    # If no faxnum is given it's assumed that cancelling the fax is ok.
    scalar(@faxnums) || do {
    printf ACCT "[%02d/%02d/%02d] %02d:%02d:%02d Fax from $host by $smbuser",
            $mo, $dy, $yr, $h, $m, $s;
        if ( $faxreceiver ) {
            print ACCT " to $faxreceiver";
            $faxdest = " " . $msg_to . " $faxreceiver"; 
        }
        print ACCT " cancelled\n";
        print SF "Fax $faxfname$faxdest ", $msg_ignored, "\n";
        close(SF);
        close(ACCT);
	if ($debug) {
	    print STDERR "No faxnumbers given. Fax cancelled. Aborting\n";
	}
        exit 0;
    };

### Spool it

    my(@fixargs, @faxargs, $fnum, $retval);

    # Arguments for faxspool call
    # --- Change this for other fax programs
    @fixargs = ("-q","-f","$user","-F","$fullname");

    # set delay if required
    if ( ($delayed ne '') && $usedelay ) {
      # Additional argument for faxspool call
      # --- Change this for other fax programs
      @fixargs = (@fixargs,"-t","$delayed");
    }

    # do the "faxnumbers in a file" handling

    my(@newfaxnums, @numfiles1, @numfiles2, @files, $file, $fnum2);

    if ($allowfile) {

	# seperate between faxnumbers and filenames
	foreach $fnum (@faxnums) {
	    if (substr($fnum,0,1) eq $filechar ) {
		push(@numfiles1, $fnum);
	    } else {
		push(@newfaxnums, $fnum);
	    }
	}
	@faxnums = @newfaxnums;

	# now process the file(s), a file can contain other filenames
	while (scalar(@numfiles1)) {
	    foreach $fnum (@numfiles1) {
		$file = substr($fnum, 1);

		# Don't read a file twice (avoid endless loop)!
		next if scalar(grep($_ eq $file, @files));
		push(@files, $file);

		# Get user's home dir, if path is not absolute
		if (substr($file,0,1) ne '/') {
		    $file = (getpwnam($smbuser))[7] . "/$file";
		}

		# Read a file

		if ($debug) {
		    print STDERR "read file $file - ";
		}

		if (open(NUMBERS,"<$file")) {
		    while (<NUMBERS>) {

			# remove \r and \n
			tr/\r\n//d;

			# remove trailing blanks
			s/^ *(.*) *$/$1/;

			# ignore comments
			next if /^#/;

			# separate between faxnumbers and filenames
			foreach $fnum2 (split(/[ ,]+/)) {
			    if (substr($fnum2,0,1) eq $filechar) {
				push(@numfiles2, $fnum2);
			    } else {
				push(@faxnums, $fnum2);
			    }
			}
		    }
		    close(NUMBERS);

		    if ($debug) {
			print STDERR "ok\n",
		    }

		} elsif ($debug) {
		    print STDERR "failed\n";
		}
	    }

	    # recurse files
	    @numfiles1 = @numfiles2;
	    @numfiles2 = ();

	}
    }

    # delete duplicate faxnumbers
    @newfaxnums = sort(@faxnums);
    @faxnums = ();
    $fnum2 = "";
    while ($fnum = shift(@newfaxnums)) {
	push(@faxnums, $fnum) if ($fnum ne $fnum2);
	$fnum2 = $fnum;
    }


    # Spool it once for every number in $faxnum
    foreach $fnum (@faxnums) {
        printf ACCT "[%02d/%02d/%02d] %02d:%02d:%02d Fax from $host by $smbuser ",
                $mo, $dy, $yr, $h, $m, $s;

        # Build faxspool arguments from above arguments, number and file
        # --- Change this for other fax programs
        @faxargs = (@fixargs,$fnum,$faxfile);

        if ( $faxreceiver ) {
            print ACCT "to $faxreceiver ($fnum)";
            $faxdest = "$faxreceiver ($fnum)";

            # Add receivers name to faxspool arguments
            # --- Change this for other fax programs
            @faxargs = ("-D","$faxreceiver",@faxargs);

        } else {
            print ACCT "to $fnum"; 
            $faxdest = "$fnum";
        }

        # Call the fax spooler
	if ($debug) {
	    print STDERR "simulate " if ($nosendondebug);
	    print STDERR "call: $faxspool ", join(" ", @faxargs), "\n";
	}
	if (!$debug || ($debug && !$nosendondebug)) {
	    system($faxspool, @faxargs);
	}
	

        # Report the state to the logfile
        $retval = $?;
	if ($debug && !$nosendondebug) {
	    print STDERR "spooling ";
	    print STDERR ($retval == 0) ? "succeeded\n" : "failed\n";
	}
        print ACCT (($retval == 0) ? " spooled\n" : " failed\n");
        print SF "Fax $faxfname an $faxdest ", 
           (($retval == 0) ? $msg_spooled : $msg_failed ), "\n";
    }

### Done
    close(SF);
    close(ACCT);
    exit 0;
